using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using NwndAdHcWaAPI.BusinessObject;
using NwndAdHcWaAPI.Models;
using NwndAdHcWaAPI.Domain;
using System.Collections.Generic;
using System.Text.Json;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace NwndAdHcWaWebAPI.ApiControllers.Base
{
     /// <summary>
     /// Base class for CategoriesApiController.  Do not make changes to this class,
     /// instead, put additional code in the CategoriesApiController class
     /// </summary>
     public class CategoriesApiControllerBase : ControllerBase
     {

         /// <summary>
         /// Inserts/Adds/Creates a new record in the database
         /// </summary>
         /// <param name="model">Pass the CategoriesModel here.  Arrives as CategoriesFields which automatically strips the data annotations from the CategoriesModel.</param>
         /// <returns>Task of IActionResult</returns>
         [HttpPost]
         public async Task<IActionResult> Insert([FromBody]CategoriesModel model, bool isForListInlineOrListCrud = false)
         {
             // add a new record
             return await AddEditCategoriesAsync(model, CrudOperation.Add, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Updates an existing record in the database by primary key.  Pass the primary key in the CategoriesModel
         /// </summary>
         /// <param name="model">Pass the CategoriesModel here.  Arrives as CategoriesFields which automatically strips the data annotations from the CategoriesModel.</param>
         /// <returns>Task of IActionResult</returns>
         [HttpPost]
         public async Task<IActionResult> Update([FromBody]CategoriesModel model, bool isForListInlineOrListCrud = false)
         {
             // update existing record
             return await AddEditCategoriesAsync(model, CrudOperation.Update, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Deletes an existing record by primary key
         /// </summary>
         /// <param name="id">CategoryID</param>
         /// <returns>Task of IActionResult</returns>
         [HttpDelete]
         public async Task<IActionResult> Delete(int id)
         {
             try
             {
                 // delete a record based on id(s)
                 await Categories.DeleteAsync(id);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {

                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message + " Stack Trace: " + ex.StackTrace);
             }
         }

         /// <summary>
         /// Deletes multiple records based on the comma-delimited ids (primary keys)
         /// </summary>
         /// <param name="ids">List of CategoryID</param>
         /// <returns>Task of IActionResult</returns>
         [HttpDelete]
         public async Task<IActionResult> DeleteMultiple(string ids)
         {
             try
             {
                 // split ids into a List
                 List<Int32> categoryIDList = ids.Split(",").Select(Int32.Parse).ToList();

                 // delete multiple records based on a list of ids (primary keys)
                 await Categories.DeleteMultipleAsync(categoryIDList);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message + " Stack Trace: " + ex.StackTrace);
             }
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectSkipAndTake(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Categories> objCategoriesCol = await Categories.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression.
         /// </summary>
         /// <param name="_search">true or false</param>
         /// <param name="nd">nd</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="page">Current page</param>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="filters">Optional.  Filters used in search</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectSkipAndTakeWithFilters(string _search, string nd, int rows, int page, string sidx, string sord, string filters = "")
         {
             int? categoryID = null;
             string categoryName = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = Newtonsoft.Json.JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "categoryid")
                         categoryID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "categoryname")
                         categoryName = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (categoryID == -1)
                     categoryID = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records based on filters
             List<Categories> objCategoriesCol = await Categories.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, rows, startRowIndex, sidx + " " + sord);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectSkipAndTakeWithTotals(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Categories> objCategoriesCol = await Categories.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         /// <param name="id">CategoryID</param>
         /// <returns>Returns one Categories record</returns>
         [HttpGet]
         public async Task<Categories> SelectByPrimaryKey(int id)
         {
             Categories objCategories = await Categories.SelectByPrimaryKeyAsync(id);
             return objCategories;
         }

         /// <summary>
         /// Gets the total number of records in the Categories table
         /// </summary>
         /// <returns>Returns the Total number of records in the Categories table</returns>
         [HttpGet]
         public async Task<int> GetRecordCount()
         {
             return await Categories.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Categories table based on search parameters
         /// </summary>
         /// <param name="categoryID">CategoryID</param>
         /// <param name="categoryName">CategoryName</param>
         /// <returns>Returns the Total number of records in the Categories table based on the search parameters</returns>
         [HttpGet]
         public Task<int> GetRecordCountDynamicWhere(int? categoryID, string categoryName)
         {
             return Categories.GetRecordCountDynamicWhereAsync(categoryID, categoryName);
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression.
         /// </summary>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <returns>Returns Categories (List of) collection.</returns>
         public async Task<List<Categories>> SelectSkipAndTake(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             return await Categories.SelectSkipAndTakeAsync(rows, startRowIndex, sortByExpression);
         }

         /// <summary>
         /// Selects records as a collection (List) of Categories sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         /// <param name="categoryID">CategoryID</param>
         /// <param name="categoryName">CategoryName</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <param name="page">Page of the grid to show</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectSkipAndTakeDynamicWhere(int? categoryID, string categoryName, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             List<Categories> objCategoriesCol = await Categories.SelectSkipAndTakeDynamicWhereAsync(categoryID, categoryName, rows, startRowIndex, sortByExpression);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Categories sorted by the sort expression.  Data returned is not sorted when sortByExpression is not passed, null, or empty.
         /// </summary>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc".  sortByExpression is not required.</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectAll(string sortByExpression = null)
         {
             List<Categories> objCategoriesCol;

             if (String.IsNullOrEmpty(sortByExpression))
             {
                 objCategoriesCol = await Categories.SelectAllAsync();
             }
             else
             {
                 sortByExpression = GetSortExpression(sortByExpression);
                 objCategoriesCol = await Categories.SelectAllAsync(sortByExpression);
             }

             return objCategoriesCol;
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Categories.
         /// </summary>
         /// <param name="categoryID">CategoryID</param>
         /// <param name="categoryName">CategoryName</param>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectAllDynamicWhere(int? categoryID, string categoryName)
         {
             List<Categories> objCategoriesCol = await Categories.SelectAllDynamicWhereAsync(categoryID, categoryName);
             return objCategoriesCol;
         }

         /// <summary>
         /// Selects CategoryID and CategoryName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         /// <returns>Returns a collection (List) of Categories</returns>
         [HttpGet]
         public async Task<List<Categories>> SelectCategoriesDropDownListData()
         {
             List<Categories> objCategoriesCol = await Categories.SelectCategoriesDropDownListDataAsync();
             return objCategoriesCol;
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, CategoryID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "CategoryID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Used when adding a new record or updating an existing record
         /// </summary>
         /// <param name="model">CategoriesModel</param>
         /// <param name="operation">Operation to Add a new record or Update an existing record</param>
         /// <param name="isForListInlineOrListCrud">Used by the razor Views with ListInline or ListCrud when true</param>
         /// <returns>Task of IActionResult</returns>
         private async Task<IActionResult> AddEditCategoriesAsync(CategoriesModel model, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             try
             {
                 Categories objCategories;

                 // create a new instance of the Categories when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    objCategories = new Categories();
                 else
                    objCategories = await Categories.SelectByPrimaryKeyAsync(model.CategoryID);

                 // assign values to the Categories instance
                 objCategories.CategoryID = model.CategoryID;
                 objCategories.CategoryName = model.CategoryName;
                 objCategories.Description = model.Description;

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await objCategories.InsertAsync();
                 else
                    await objCategories.UpdateAsync();

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message + " Stack Trace: " + ex.StackTrace);
             }
         }
     }
}
