using Microsoft.AspNetCore.Mvc;
using System;
using System.Linq;
using NwndAdHcWaAPI.BusinessObject;
using NwndAdHcWaAPI.Models;
using NwndAdHcWaAPI.Domain;
using System.Collections.Generic;
using System.Text.Json;
using System.Net;
using System.Net.Http;
using System.Threading.Tasks;

namespace NwndAdHcWaWebAPI.ApiControllers.Base
{
     /// <summary>
     /// Base class for SuppliersApiController.  Do not make changes to this class,
     /// instead, put additional code in the SuppliersApiController class
     /// </summary>
     public class SuppliersApiControllerBase : ControllerBase
     {

         /// <summary>
         /// Inserts/Adds/Creates a new record in the database
         /// </summary>
         /// <param name="model">Pass the SuppliersModel here.  Arrives as SuppliersFields which automatically strips the data annotations from the SuppliersModel.</param>
         /// <returns>Task of IActionResult</returns>
         [HttpPost]
         public async Task<IActionResult> Insert([FromBody]SuppliersModel model, bool isForListInlineOrListCrud = false)
         {
             // add a new record
             return await AddEditSuppliersAsync(model, CrudOperation.Add, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Updates an existing record in the database by primary key.  Pass the primary key in the SuppliersModel
         /// </summary>
         /// <param name="model">Pass the SuppliersModel here.  Arrives as SuppliersFields which automatically strips the data annotations from the SuppliersModel.</param>
         /// <returns>Task of IActionResult</returns>
         [HttpPost]
         public async Task<IActionResult> Update([FromBody]SuppliersModel model, bool isForListInlineOrListCrud = false)
         {
             // update existing record
             return await AddEditSuppliersAsync(model, CrudOperation.Update, isForListInlineOrListCrud);
         }

         /// <summary>
         /// Deletes an existing record by primary key
         /// </summary>
         /// <param name="id">SupplierID</param>
         /// <returns>Task of IActionResult</returns>
         [HttpDelete]
         public async Task<IActionResult> Delete(int id)
         {
             try
             {
                 // delete a record based on id(s)
                 await Suppliers.DeleteAsync(id);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {

                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message + " Stack Trace: " + ex.StackTrace);
             }
         }

         /// <summary>
         /// Deletes multiple records based on the comma-delimited ids (primary keys)
         /// </summary>
         /// <param name="ids">List of SupplierID</param>
         /// <returns>Task of IActionResult</returns>
         [HttpDelete]
         public async Task<IActionResult> DeleteMultiple(string ids)
         {
             try
             {
                 // split ids into a List
                 List<Int32> supplierIDList = ids.Split(",").Select(Int32.Parse).ToList();

                 // delete multiple records based on a list of ids (primary keys)
                 await Suppliers.DeleteMultipleAsync(supplierIDList);

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message + " Stack Trace: " + ex.StackTrace);
             }
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectSkipAndTake(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression.
         /// </summary>
         /// <param name="_search">true or false</param>
         /// <param name="nd">nd</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="page">Current page</param>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="filters">Optional.  Filters used in search</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectSkipAndTakeWithFilters(string _search, string nd, int rows, int page, string sidx, string sord, string filters = "")
         {
             int? supplierID = null;
             string companyName = String.Empty;
             string contactName = String.Empty;
             string contactTitle = String.Empty;
             string address = String.Empty;
             string city = String.Empty;
             string region1 = String.Empty;
             string postalCode = String.Empty;
             string country = String.Empty;
             string phone = String.Empty;
             string fax = String.Empty;

             if (!String.IsNullOrEmpty(filters))
             {
                 // deserialize filters and get values being searched
                 var jsonResult = Newtonsoft.Json.JsonConvert.DeserializeObject<Dictionary<string, dynamic>>(filters);

                 foreach (var rule in jsonResult["rules"])
                 {
                     if (rule["field"].Value.ToLower() == "supplierid")
                         supplierID = Convert.ToInt32(rule["data"].Value);

                     if (rule["field"].Value.ToLower() == "companyname")
                         companyName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contactname")
                         contactName = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "contacttitle")
                         contactTitle = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "address")
                         address = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "city")
                         city = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "region1")
                         region1 = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "postalcode")
                         postalCode = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "country")
                         country = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "phone")
                         phone = rule["data"].Value;

                     if (rule["field"].Value.ToLower() == "fax")
                         fax = rule["data"].Value;

                 }

                 // sometimes jqgrid assigns a -1 to numeric fields when no value is assigned
                 // instead of assigning a null, we'll correct this here
                 if (supplierID == -1)
                     supplierID = null;

             }

             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records based on filters
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sidx + " " + sord);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression.
         /// </summary>
         /// <param name="sidx">Field to sort.  Can be an empty string.</param>
         /// <param name="sord">asc or an empty string = ascending.  desc = descending</param>
         /// <param name="page">Current page</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectSkipAndTakeWithTotals(string sidx, string sord, int page, int rows)
         {
             // get the index where to start retrieving records from
             // 0 = starts from the beggining, 10 means skip the first 10 records and start from record 11
             int startRowIndex = ((page * rows) - rows);

             // get records
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSkipAndTakeAsync(rows, startRowIndex, sidx + " " + sord);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects a record by primary key(s)
         /// </summary>
         /// <param name="id">SupplierID</param>
         /// <returns>Returns one Suppliers record</returns>
         [HttpGet]
         public async Task<Suppliers> SelectByPrimaryKey(int id)
         {
             Suppliers objSuppliers = await Suppliers.SelectByPrimaryKeyAsync(id);
             return objSuppliers;
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table
         /// </summary>
         /// <returns>Returns the Total number of records in the Suppliers table</returns>
         [HttpGet]
         public async Task<int> GetRecordCount()
         {
             return await Suppliers.GetRecordCountAsync();
         }

         /// <summary>
         /// Gets the total number of records in the Suppliers table based on search parameters
         /// </summary>
         /// <param name="supplierID">SupplierID</param>
         /// <param name="companyName">CompanyName</param>
         /// <param name="contactName">ContactName</param>
         /// <param name="contactTitle">ContactTitle</param>
         /// <param name="address">Address</param>
         /// <param name="city">City</param>
         /// <param name="region1">Region1</param>
         /// <param name="postalCode">PostalCode</param>
         /// <param name="country">Country</param>
         /// <param name="phone">Phone</param>
         /// <param name="fax">Fax</param>
         /// <returns>Returns the Total number of records in the Suppliers table based on the search parameters</returns>
         [HttpGet]
         public Task<int> GetRecordCountDynamicWhere(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             return Suppliers.GetRecordCountDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression.
         /// </summary>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <returns>Returns Suppliers (List of) collection.</returns>
         public async Task<List<Suppliers>> SelectSkipAndTake(int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             return await Suppliers.SelectSkipAndTakeAsync(rows, startRowIndex, sortByExpression);
         }

         /// <summary>
         /// Selects records as a collection (List) of Suppliers sorted by the sortByExpression starting from the startRowIndex, based on the search parameters
         /// </summary>
         /// <param name="supplierID">SupplierID</param>
         /// <param name="companyName">CompanyName</param>
         /// <param name="contactName">ContactName</param>
         /// <param name="contactTitle">ContactTitle</param>
         /// <param name="address">Address</param>
         /// <param name="city">City</param>
         /// <param name="region1">Region1</param>
         /// <param name="postalCode">PostalCode</param>
         /// <param name="country">Country</param>
         /// <param name="phone">Phone</param>
         /// <param name="fax">Fax</param>
         /// <param name="rows">Number of rows to retrieve</param>
         /// <param name="startRowIndex">Zero-based.  Row index where to start taking rows from</param>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc"</param>
         /// <param name="page">Page of the grid to show</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectSkipAndTakeDynamicWhere(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax, int rows, int startRowIndex, string sortByExpression)
         {
             sortByExpression = GetSortExpression(sortByExpression);
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSkipAndTakeDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax, rows, startRowIndex, sortByExpression);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects all records as a collection (List) of Suppliers sorted by the sort expression.  Data returned is not sorted when sortByExpression is not passed, null, or empty.
         /// </summary>
         /// <param name="sortByExpression">Field to sort and sort direction.  E.g. "FieldName asc" or "FieldName desc".  sortByExpression is not required.</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectAll(string sortByExpression = null)
         {
             List<Suppliers> objSuppliersCol;

             if (String.IsNullOrEmpty(sortByExpression))
             {
                 objSuppliersCol = await Suppliers.SelectAllAsync();
             }
             else
             {
                 sortByExpression = GetSortExpression(sortByExpression);
                 objSuppliersCol = await Suppliers.SelectAllAsync(sortByExpression);
             }

             return objSuppliersCol;
         }

         /// <summary>
         /// Selects records based on the passed filters as a collection (List) of Suppliers.
         /// </summary>
         /// <param name="supplierID">SupplierID</param>
         /// <param name="companyName">CompanyName</param>
         /// <param name="contactName">ContactName</param>
         /// <param name="contactTitle">ContactTitle</param>
         /// <param name="address">Address</param>
         /// <param name="city">City</param>
         /// <param name="region1">Region1</param>
         /// <param name="postalCode">PostalCode</param>
         /// <param name="country">Country</param>
         /// <param name="phone">Phone</param>
         /// <param name="fax">Fax</param>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectAllDynamicWhere(int? supplierID, string companyName, string contactName, string contactTitle, string address, string city, string region1, string postalCode, string country, string phone, string fax)
         {
             List<Suppliers> objSuppliersCol = await Suppliers.SelectAllDynamicWhereAsync(supplierID, companyName, contactName, contactTitle, address, city, region1, postalCode, country, phone, fax);
             return objSuppliersCol;
         }

         /// <summary>
         /// Selects SupplierID and CompanyName columns for use with a DropDownList web control, ComboBox, CheckedBoxList, ListView, ListBox, etc
         /// </summary>
         /// <returns>Returns a collection (List) of Suppliers</returns>
         [HttpGet]
         public async Task<List<Suppliers>> SelectSuppliersDropDownListData()
         {
             List<Suppliers> objSuppliersCol = await Suppliers.SelectSuppliersDropDownListDataAsync();
             return objSuppliersCol;
         }

         /// <summary>
         /// Gets the default sort expression when no sort expression is provided
         /// </summary>
         private string GetSortExpression(string sortByExpression)
         {
             // when no sort expression is provided, SupplierID is set as the default in ascending order
             // for ascending order, "asc" is not needed, so it is removed
             if (String.IsNullOrEmpty(sortByExpression) || sortByExpression == " asc")
                 sortByExpression = "SupplierID";
             else if (sortByExpression.Contains(" asc"))
                 sortByExpression = sortByExpression.Replace(" asc", "");

             return sortByExpression;
         }

         /// <summary>
         /// Used when adding a new record or updating an existing record
         /// </summary>
         /// <param name="model">SuppliersModel</param>
         /// <param name="operation">Operation to Add a new record or Update an existing record</param>
         /// <param name="isForListInlineOrListCrud">Used by the razor Views with ListInline or ListCrud when true</param>
         /// <returns>Task of IActionResult</returns>
         private async Task<IActionResult> AddEditSuppliersAsync(SuppliersModel model, CrudOperation operation, bool isForListInlineOrListCrud = false)
         {
             try
             {
                 Suppliers objSuppliers;

                 // create a new instance of the Suppliers when adding a new record
                 // or, retrieve the record that needs to be updated
                 if (operation == CrudOperation.Add)
                    objSuppliers = new Suppliers();
                 else
                    objSuppliers = await Suppliers.SelectByPrimaryKeyAsync(model.SupplierID);

                 // assign values to the Suppliers instance
                 objSuppliers.SupplierID = model.SupplierID;
                 objSuppliers.CompanyName = model.CompanyName;
                 objSuppliers.ContactName = model.ContactName;
                 objSuppliers.ContactTitle = model.ContactTitle;
                 objSuppliers.Address = model.Address;
                 objSuppliers.City = model.City;
                 objSuppliers.Region1 = model.Region1;
                 objSuppliers.PostalCode = model.PostalCode;
                 objSuppliers.Country = model.Country;
                 objSuppliers.Phone = model.Phone;
                 objSuppliers.Fax = model.Fax;
                 objSuppliers.HomePage = model.HomePage;

                 // save the new record, or the updated values of the current record
                 if (operation == CrudOperation.Add)
                    await objSuppliers.InsertAsync();
                 else
                    await objSuppliers.UpdateAsync();

                 // everthing went well
                 return Ok();
             }
             catch (Exception ex)
             {
                 // something went wrong
                 return BadRequest("Error Message: " + ex.Message + " Stack Trace: " + ex.StackTrace);
             }
         }
     }
}
